#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace HeuristicLab.Core {
  /// <summary>
  /// Interface for a container of variables.
  /// </summary>
  public interface IVariablesContainer : IItem {
    /// <summary>
    /// Gets the contained varibles.
    /// </summary>
    ICollection<IVariable> Variables { get; }

    /// <summary>
    /// Gets a variable with the given <paramref name="name"/>.
    /// </summary>
    /// <param name="name">The name of the variable.</param>
    /// <returns>The variable with the specified name.</returns>
    IVariable GetVariable(string name);
    /// <summary>
    /// Adds the specified variable to the curent instance.
    /// </summary>
    /// <param name="variable">The variable to add.</param>
    void AddVariable(IVariable variable);
    /// <summary>
    /// Deletes a variable with the specified <paramref name="name"/> from the current instance.
    /// </summary>
    /// <param name="name">The name of the variable to delete.</param>
    void RemoveVariable(string name);

    /// <summary>
    /// Occurs when a variable has been added to the current instance.
    /// </summary>
    event EventHandler<VariableEventArgs> VariableAdded;
    /// <summary>
    /// Occurs when a variable has been deleted from the current instance.
    /// </summary>
    event EventHandler<VariableEventArgs> VariableRemoved;
  }
}
